class SpeechBalloonsController < ApplicationController
  before_filter :authenticate_user!, :only => [:index, :show]
  before_filter :authenticate_admin!, :only => [:list, :browse, :create, :update, :destroy]

  private
  
  def validate_param(ft)
    res = nil
    bl = ft[:size_count].to_i
    tl = ft[:tail_count].to_i
    if bl * tl == ft[:balloon_templates_attributes].size
      flag = []
      bl.times do |i|
        flag[i] = []
        tl.times do |j|
          flag[i][j] = true
        end
      end
      ft[:balloon_templates_attributes].each do |k, t|
        flag[t[:size].to_i - 1][t[:tail].to_i - 1] = nil
      end
      if flag.flatten.compact.size == 0
      else
        res = "invalid variation"
      end
    else
      res = "invalid template size"
    end
    res
  end
  
  public
  # GET /speech_balloons
  # GET /speech_balloons.json
  def index
    @speech_balloons = SpeechBalloon.all

    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @speech_balloons }
    end
  end

  # GET /speech_balloons/1
  # GET /speech_balloons/1.json
  def show
    @speech_balloon = SpeechBalloon.find(params[:id], include: {:balloon_templates => :speech_templates})

    respond_to do |format|
      format.html # show.html.erb
      format.json {
        render :json => @speech_balloon.to_json(include: {
          :balloon_templates => {:include => :speech_templates}
        })
      }
      format.jsonp {
        render :json => "callback(" + @speech_balloon.to_json(include: {
          :balloon_templates => {:include => :speech_templates}
        }) + ")"
      }
    end
  end

  def list
    @speech_balloons = SpeechBalloon.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @speech_balloons }
    end
  end

  def browse
    @speech_balloon = SpeechBalloon.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @speech_balloon }
    end
  end

  # POST /speech_balloons
  # POST /speech_balloons.json
  def create
    @speech_balloon = SpeechBalloon.new(params[:speech_balloon])
    if em = validate_param(params[:speech_balloon])
      respond_to do |format|
        format.json { render json: em, status: :unprocessable_entity }
      end
    else
      respond_to do |format|
        if @speech_balloon.save
          format.html { redirect_to @speech_balloon, notice: 'Speech balloon was successfully created.' }
          format.json { render json: @speech_balloon, status: :created, location: @speech_balloon }
        else
          format.html { render action: "new" }
          format.json { render json: @speech_balloon.errors, status: :unprocessable_entity }
        end
      end
    end
  end

  # PUT /speech_balloons/1
  # PUT /speech_balloons/1.json
  def update
    @speech_balloon = SpeechBalloon.find(params[:id])

    respond_to do |format|
      if @speech_balloon.update_attributes(params[:speech_balloon])
        format.html { redirect_to @speech_balloon, notice: 'Speech balloon was successfully updated.' }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @speech_balloon.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /speech_balloons/1
  # DELETE /speech_balloons/1.json
  def destroy
    @speech_balloon = SpeechBalloon.find(params[:id])
    @speech_balloon.destroy

    respond_to do |format|
      format.html { redirect_to speech_balloons_url }
      format.json { head :ok }
    end
  end
end
