#実素材
class Picture < ActiveRecord::Base
  belongs_to :original_picture
  belongs_to :license
  belongs_to :artist
  has_one :resource_picture
  
  validates :original_picture_id, :presence => true, :numericality => true, :existence => true
  validates :revision, :presence => true, :numericality => true
  validates :ext, :presence => true, :length => {:maximum => 4}, :inclusion => {:in => ['png', 'jpeg', 'gif']}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :filesize, :presence => true, :numericality => {:greater_than => 0, :less_than_or_equal_to => 2000000}, :natural_number => true
  validates :license_id, :presence => true, :numericality => true, :existence => true
  validates :artist_id, :presence => true, :numericality => true, :existence => true
  validates :artist_name, :presence => true
  validates :classname, :presence => true, :length => {:maximum => 50}
  
  def dext
    self.ext.downcase
  end
  
  def filename
    "#{self.id}.#{self.dext}"
  end
  
  def gifname
    "#{self.id}.gif"
  end
  
  def mime_type
    "image/#{self.dext}"
  end
  
  def new_revision
    Picture.maximum(:revision, :conditions => ['original_picture_id = ?', self.original_picture_id]).to_i + 1
  end
  
  def v(rimg)
    rimg.flip.to_blob
  end
  
  def h(rimg)
    rimg.flop.to_blob
  end
  
  def vh(rimg)
    rimg.flip.flop.to_blob
  end
  
  def to_gif?
    self.dext == 'png' and self.flag_gif_convert >= 0
  end
  
  def self.png_to_gif(data)
    res = nil
    begin
      mgk = Magick::Image.from_blob(data).shift
      mgk.format = 'gif'
      res = mgk
    rescue
      res = false
    end
    res
  end
  
  def subdirs
    flag_reverse < 0 ? [''] : ['', 'v', 'h', 'vh']
  end
  
  def copy_data(rp)
    attr = {:width => rp.width, :height => rp.height, :ext => rp.ext, :filesize => rp.filesize, 
      :original_picture_id => rp.original_picture_id, :license_id => rp.license_id, :artist_id => rp.artist_id, 
      :artist_name => rp.artist_name, :classname => rp.classname, :credit => rp.credit, :settings => rp.settings
    }
    self.attributes = attr
  end
  
  def store(mgk)
    res = false
    self.revision = self.new_revision
    if res = self.save
      if res = self.store_picture(mgk, self.filename)
        if self.to_gif?
          if gifmgk = Picture.png_to_gif(mgk.to_blob)
            res = self.store_picture(gifmgk, self.gifname)
          else
            res = false
          end
        end
      end
    end
    res
  end
  
  def store_picture(mgk, fn)
    res = false
    subdirs.each do |d|
      picdata = d.empty? ? mgk.to_blob : self.__send__(d, mgk)
      res = PictureIO.picture_io.put(picdata, fn, d)
      break unless res
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.resource_picture_io.get self.filename, subdir
  end
  
  def self.show pid
    r = Picture.find(pid)
#    raise ActiveRecord::Forbidden unless c.visible?(au)
    r
  end
  
  def credit_template
    "#{self.classname.tableize}/attributes/credit"
  end
  
  def credit_data
    begin
      @credit_data = JSON.parse(self.credit) unless @credit_data
    rescue 
    end
    @credit_data
  end
  
  def flags
    return @flags if @flags
    begin
      @flags = JSON.parse(self.settings)
    rescue 
    end
    @flags = {} unless @flags
    @flags
  end
  
  def flags=(s)
    @flags = s
  end
  
  def flag_open
    @flag_open = (flags["open"] || -1) unless @flag_open
    @flag_open
  end
  
  def flag_commercial
    @flag_commercial = (flags["commercial"] || -1) unless @flag_commercial
    @flag_commercial
  end
  
  def flag_official
    @flag_official = (flags["official"] || -1) unless @flag_official
    @flag_official
  end
  
  def flag_attribution
    @flag_attribution = (flags["attribution"] || -1) unless @flag_attribution
    @flag_attribution
  end
  
  def flag_derive
    @flag_derive = (flags["derive"] || -1) unless @flag_derive
    @flag_derive
  end
  
  def flag_thumbnail
    @flag_thumbnail = (flags["thumbnail"] || -1) unless @flag_thumbnail
    @flag_thumbnail
  end
  
  def flag_gif_convert
    @flag_gif_convert = (flags["gif_convert"] || -1) unless @flag_gif_convert
    @flag_gif_convert
  end
  
  def flag_reverse
    @flag_reverse = (flags["reverse"] || -1) unless @flag_reverse
    @flag_reverse
  end
  
  def flag_resize
    @flag_resize = (flags["resize"] || -1) unless @flag_resize
    @flag_resize
  end
  
  def flag_sync_vh
    @flag_sync_vh = (flags["sync_vh"] || -1) unless @flag_sync_vh
    @flag_sync_vh
  end
  
  def flag_overlap
    @flag_overlap = (flags["overlap"] || -1) unless @flag_overlap
    @flag_overlap
  end
  
end
