# -*- encoding: utf-8 -*-
#ライセンスグループ
require 'spec_helper'

describe LicenseGroup do
  before do
    #テストデータを用意してね
    @f = Rails.root + 'spec/json/license_group.json'
    @t = File.open(@f, 'r').read
    @j = JSON.parse @t
    @fs = Rails.root + 'spec/json/license_groups.json'
    @fes = Rails.root + 'spec/json/invalid_license_groups.json'
  end
  describe '検証に於いて' do
    before do
    end
    
    it 'オーソドックスなデータなら通る' do
      @lg = FactoryGirl.build :license_group
      @lg.should be_valid
    end
    
    context 'nameを検証するとき' do
      before do
        @lg = FactoryGirl.build :license_group
      end
      it 'テストデータの確認' do
        @lg.name = 'a'*50
        @lg.should be_valid
      end
      it 'nullなら失敗する' do
        @lg.name = ''
        @lg.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @lg.name = 'a'*51
        @lg.should_not be_valid
      end
      it '重複していたら失敗する' do
        l = FactoryGirl.create :license_group
        @lg.should_not be_valid
      end
    end
    context 'classnameを検証するとき' do
      before do
        @lg = FactoryGirl.build :license_group
      end
      it 'テストデータの確認' do
        @lg.classname = 'a'*50
        @lg.should be_valid
      end
      it 'nullなら失敗する' do
        @lg.classname = ''
        @lg.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @lg.classname = 'a'*51
        @lg.should_not be_valid
      end
    end
    context 'captionを検証するとき' do
      before do
        @lg = FactoryGirl.build :license_group
      end
      it 'テストデータの確認' do
        @lg.caption = 'a'*30
        @lg.should be_valid
      end
      it 'nullなら失敗する' do
        @lg.caption = ''
        @lg.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @lg.caption = 'a'*51
        @lg.should_not be_valid
      end
    end
    context 'urlを検証するとき' do
      before do
        @lg = FactoryGirl.build :license_group
      end
      it 'テストデータの確認' do
        @lg.url = 'http://test.com/'
        @lg.should be_valid
      end
      it 'nullなら失敗する' do
        @lg.url = ''
        @lg.should_not be_valid
      end
      it '201文字以上なら失敗する' do
        @lg.url = 'a'*201
        @lg.should_not be_valid
      end
      it 'url形式でないなら失敗する' do
        @lg.url = 'aaaaaaa'
        @lg.should_not be_valid
      end
    end
  end
  
  describe '更新に於いて' do
    before do
      @n = @j.keys.first
      @a = @j.values.first
    end
    context 'つつがなく終わるとき' do
      it 'データ更新準備を依頼する' do
        LicenseGroup.stub(:modify_object).with(any_args).and_return(LicenseGroup.new)
        LicenseGroup.should_receive(:modify_object).with(any_args).exactly(1)
        License.stub(:stores).with(any_args).and_return(0)
        LicenseGroup.any_instance.stub(:save).with(any_args).and_return(true)
        r = LicenseGroup.store(@n, @a)
      end
      it 'ライセンスに複数の更新を依頼する' do
        LicenseGroup.stub(:modify_object).with(any_args).and_return(LicenseGroup.new)
        License.stub(:stores).with(any_args).and_return(0)
        License.should_receive(:stores).with(any_args).exactly(1)
        LicenseGroup.any_instance.stub(:save).with(any_args).and_return(true)
        r = LicenseGroup.store(@n, @a)
      end
      it '保存を依頼する' do
        LicenseGroup.stub(:modify_object).with(any_args).and_return(LicenseGroup.new)
        License.stub(:stores).with(any_args).and_return(0)
        LicenseGroup.any_instance.stub(:save).with(any_args).and_return(true)
        LicenseGroup.any_instance.should_receive(:save).with(any_args).exactly(1)
        r = LicenseGroup.store(@n, @a)
      end
      it 'オブジェクトを返す' do
        r = LicenseGroup.store(@n, @a)
        r.is_a?(LicenseGroup).should be_true
        r.name.should eq @n
        r.url.should eq @a["url"]
      end
      it 'カラム値からlicenses_attributesが削除されている' do
        @a["licenses_attributes"].should_not be_nil
        r = LicenseGroup.store(@n, @a)
        @a["licenses_attributes"].should be_nil
      end
    end
    context 'ライセンス複数更新が失敗するとき' do
      before do
        LicenseGroup.stub(:modify_object).with(any_args).and_return(LicenseGroup.new)
        License.stub(:stores).with(any_args).and_return(1)
        LicenseGroup.any_instance.stub(:save).with(any_args).and_return(true)
      end
      it '全体エラーメッセージがセットされている' do
        r = LicenseGroup.store(@n, @a)
        r.errors[:base].should_not be_blank
      end
      it 'ライセンスが作成されていない' do
        lambda {
          r = LicenseGroup.store(@n, @a)
        }.should_not change License, :count
      end
      it 'ライセンスグループが作成されていない' do
        lambda {
          r = LicenseGroup.store(@n, @a)
        }.should_not change LicenseGroup, :count
      end
    end
  end
  
  describe 'インポートに於いて' do
    before do
    end
    context 'つつがなく終わるとき' do
      it 'ファイルインポートを依頼する' do
        LicenseGroup.should_receive(:import_file).with(any_args).exactly(1)
        LicenseGroup.stub(:import_file).with(any_args).and_return([])
        LicenseGroup.import(@f)
      end
      it 'ライセンスグループ更新を一回依頼する' do
        LicenseGroup.stub(:store).with(any_args).and_return(LicenseGroup.new)
        LicenseGroup.should_receive(:store).with(any_args).exactly(1)
        LicenseGroup.import(@f)
      end
      it 'ライセンスグループが追加される' do
        lambda {
          LicenseGroup.import(@f)
        }.should change LicenseGroup, :count
      end
      it 'ライセンスが追加される' do
        lambda {
          LicenseGroup.import(@f)
        }.should change License, :count
      end
      it '[]を返す' do
        r = LicenseGroup.import(@f)
        r.should eq []
      end
    end
    context '複数データがつつがなく終わるとき' do
      it 'ライセンスグループ更新を二回依頼する' do
        LicenseGroup.stub(:store).with(any_args).and_return(LicenseGroup.new)
        LicenseGroup.should_receive(:store).with(any_args).exactly(2)
        LicenseGroup.import(@fs)
      end
      it 'ライセンスグループが二個追加される' do
        lambda {
          r = LicenseGroup.import(@fs)
        }.should change(LicenseGroup, :count).by 2
      end
      it 'ライセンスが追加される' do
        lambda {
          r = LicenseGroup.import(@fs)
        }.should change(License, :count)
      end
      it '[]を返す' do
        r = LicenseGroup.import(@fs)
        r.should eq []
      end
    end
    context 'ライセンスグループ作成に失敗したとき' do
      before do
        LicenseGroup.any_instance.stub(:save).with(any_args).and_return(false)
        LicenseGroup.any_instance.stub(:valid?).with(any_args).and_return(false)
      end
      it 'ライセンスグループの数に変化がない' do
        lambda {
          LicenseGroup.import(@f)
        }.should_not change LicenseGroup, :count
      end
      it '配列を返す' do
        r = LicenseGroup.import(@f)
        r.is_a?(Array).should be_true
      end
      it '配列の中身は一件' do
        r = LicenseGroup.import(@f)
        r.should have(1).items
      end
      it 'ライセンスグループオブジェクトが入っている' do
        r = LicenseGroup.import(@f)
        r.first.is_a?(LicenseGroup).should be_true
      end
    end
    context '複数のライセンスグループ作成に失敗したとき' do
      #三件中、二件の失敗、一件を成功させ、成功データは戻り値に含まないことを確認する
      it 'ライセンスグループの数に変化がない' do
        lambda {
          LicenseGroup.import(@fes)
        }.should_not change LicenseGroup, :count
      end
      it '途中で保存に失敗しても全件更新依頼する' do
        LicenseGroup.stub(:store).with(any_args).and_return(LicenseGroup.new)
        LicenseGroup.should_receive(:store).with(any_args).exactly(3)
        LicenseGroup.import(@fes)
      end
      it '配列を返す' do
        r = LicenseGroup.import(@fes)
        r.is_a?(Array).should be_true
      end
      it '配列の中身は2件' do
        r = LicenseGroup.import(@fes)
        r.should have(2).items
      end
      it '配列の中身は失敗したライセンスグループオブジェクトが入っている' do
        r = LicenseGroup.import(@fes)
        r[0].is_a?(LicenseGroup).should be_true
        r[0]["name"].should eq 'UnknownUrl'
        r[1].is_a?(LicenseGroup).should be_true
        r[1]["name"].should eq 'UnknownClassname'
      end
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @lg = FactoryGirl.create :license_group
    end
    it '指定のコマを返す' do
      l = LicenseGroup.show @lg.id
      l.should eq @lg
    end
  end
  describe '関連テーブルプションに於いて' do
    context 'オプションがないとき' do
      it 'ライセンスを含んでいる' do
        r = LicenseGroup.show_include_opt
        r.has_key?(:licenses).should be_true
      end
    end
  end
  describe 'json単体出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = LicenseGroup.show_json_include_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = LicenseGroup.show_json_include_opt[:include]
      r.should have(1).items
    end
    it 'ライセンスを含んでいる' do
      r = LicenseGroup.show_json_include_opt[:include]
      r.has_key?(:licenses).should be_true
    end
  end
  describe '一覧取得に於いて' do
    before do
      @lg = FactoryGirl.create :license_group, :name => "1"
    end
    it 'リストを返す' do
      l = LicenseGroup.list
      l.should eq [@lg]
    end
    it '名前順で並んでいる' do
      @lg2 = FactoryGirl.create :license_group, :name => "5", :url => 'http://test.ptn/10'
      l = LicenseGroup.list
      l.should eq [@lg, @lg2]
    end
  end
  describe 'list関連テーブルプションに於いて' do
    it 'includeキーを含んでいる' do
      r = LicenseGroup.list_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = LicenseGroup.list_opt[:include]
      r.should have(1).items
    end
    it 'ライセンスを含んでいる' do
      r = LicenseGroup.list_opt[:include]
      r.has_key?(:licenses).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = LicenseGroup.list_json_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = LicenseGroup.list_json_opt[:include]
      r.should have(1).items
    end
    it 'ライセンスを含んでいる' do
      r = LicenseGroup.list_json_opt[:include]
      r.has_key?(:licenses).should be_true
    end
  end
  
end
