# -*- encoding: utf-8 -*-
require 'spec_helper'
#コマ
describe Panel do
  before do
    FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = @other_user.author
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
    @sbt = FactoryGirl.create :speech_balloon_template
  end
  
  describe '検証に於いて' do
    before do
    end
    
    it 'オーソドックスなデータなら通る' do
      @panel = FactoryGirl.build :panel, :author_id => @author.id
      @panel.should be_valid
    end
    
    context 'widthを検証するとき' do
      before do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
      end
      it 'テストデータの確認' do
        @panel.width = 1
        @panel.should be_valid
      end
      it 'nullなら失敗する' do
        @panel.width = nil
        @panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @panel.width = 'a'
        @panel.should_not be_valid
      end
      it '0なら失敗する' do
        @panel.width = '0'
        @panel.should_not be_valid
      end
      it '負でも失敗する' do
        @panel.width = -1
        @panel.should_not be_valid
      end
    end
    context 'heightを検証するとき' do
      before do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
      end
      it 'テストデータの確認' do
        @panel.height = '1'
        @panel.should be_valid
      end
      it 'nullなら失敗する' do
        @panel.height = nil
        @panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @panel.height = 'a'
        @panel.should_not be_valid
      end
      it '0なら失敗する' do
        @panel.height = '0'
        @panel.should_not be_valid
      end
      it '負でも失敗する' do
        @panel.height = -1
        @panel.should_not be_valid
      end
    end
    context 'borderを検証するとき' do
      before do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
      end
      it 'テストデータの確認' do
        @panel.border = '1'
        @panel.should be_valid
      end
      it 'nullなら失敗する' do
        @panel.border = nil
        @panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @panel.border = 'a'
        @panel.should_not be_valid
      end
      it '負なら失敗する' do
        @panel.border = '-1'
        @panel.should_not be_valid
      end
      it '0なら通る' do
        @panel.border = 0
        @panel.should be_valid
      end
    end
    context 'xを検証するとき' do
      before do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
      end
      it 'テストデータの確認' do
        @panel.x = '1'
        @panel.should be_valid
      end
      it '数値でなければ失敗する' do
        @panel.x = 'a'
        @panel.should_not be_valid
      end
      it '0なら通る' do
        @panel.x = '0'
        @panel.should be_valid
      end
      it '負でも通る' do
        @panel.x = -1
        @panel.should be_valid
      end
    end
    context 'yを検証するとき' do
      before do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
      end
      it 'テストデータの確認' do
        @panel.y = '1'
        @panel.should be_valid
      end
      it '数値でなければ失敗する' do
        @panel.y = 'a'
        @panel.should_not be_valid
      end
      it '0なら通る' do
        @panel.y = '0'
        @panel.should be_valid
      end
      it '負でも通る' do
        @panel.y = -1
        @panel.should be_valid
      end
    end
    context 'zを検証するとき' do
      before do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
      end
      it 'テストデータの確認' do
        @panel.z = '1'
        @panel.should be_valid
      end
      it '数値でなければ失敗する' do
        @panel.z = 'a'
        @panel.should_not be_valid
      end
      it '0なら失敗する' do
        @panel.z = '0'
        @panel.should_not be_valid
      end
      it '負なら失敗する' do
        @panel.z = -1
        @panel.should_not be_valid
      end
    end
    context 'author_idを検証するとき' do
      before do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
      end
      it 'テストデータの確認' do
        @panel.author_id = @author.id
        @panel.should be_valid
      end
      it 'nullなら失敗する' do
        @panel.author_id = nil
        @panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @panel.author_id = 'a'
        @panel.should_not be_valid
      end
      it '存在する絵師でなければ失敗する' do
        @panel.author_id = 0
        @panel.should_not be_valid
      end
    end
    context '全体を検証するとき' do
      before do
        @panel = FactoryGirl.create :panel, :author_id => @author.id
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    before do
      @panel = FactoryGirl.build :panel, :author_id => @author.id
    end
    it 'borderは2を補充する' do
      @panel.border = nil
      @panel.supply_default
      @panel.border.should eq 2
    end
  end
  
  describe '上書き補充に於いて' do
    before do
      @panel = FactoryGirl.build :panel, :author_id => @author.id
    end
    context 'author_idを補充' do
      it 'ログイン中の作家idを補充する' do
        @panel.author_id = nil
        @panel.overwrite @author
        @panel.author_id.should eq @author.id
      end
    end
    
  end
  
  describe '作者判定に於いて' do
    before do
    end
    it '自作のコマならyes' do
      panel = FactoryGirl.create :panel, :author_id => @author.id
      panel.own?(@author).should == true
    end
    it '他人のコマならno' do
      panel = FactoryGirl.create :panel, :author_id => @author.id
      panel.own?(@other_author).should == false
    end
    it '作家が不明ならno' do
      panel = FactoryGirl.create :panel, :author_id => @author.id
      panel.own?(nil).should == false
    end
  end
  describe '閲覧許可に於いて' do
    before do
    end
    it '自作のコマを見るときは許可する' do
      Panel.any_instance.stub(:own?).and_return(true)
      Panel.any_instance.stub(:publish?).and_return(true)
      panel = FactoryGirl.create :panel, :author_id => @author.id
      panel.visible?(@author).should == true
    end
    it '自作のコマは非公開でも許可する' do
      Panel.any_instance.stub(:own?).and_return(true)
      Panel.any_instance.stub(:publish?).and_return(false)
      panel = FactoryGirl.create :panel, :author_id => @author.id
      panel.visible?(@author).should == true
    end
    it '他人のコマでも公開なら許可する' do
      Panel.any_instance.stub(:own?).and_return(false)
      Panel.any_instance.stub(:publish?).and_return(true)
      panel = FactoryGirl.create :panel, :author_id => @author.id
      panel.visible?(@author).should == true
    end
    it '他人のコマで非公開なら許可しない' do
      Panel.any_instance.stub(:own?).and_return(false)
      Panel.any_instance.stub(:publish?).and_return(false)
      panel = FactoryGirl.create :panel, :author_id => @author.id
      panel.visible?(@author).should == false
    end
  end
  describe '単体取得に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    it '指定のコマを返す' do
      Panel.any_instance.stub(:visible?).and_return(true)
      pl = Panel.show @panel.id, @author
      pl.should eq @panel
    end
    context '他人の非公開コミックのコマを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        Panel.any_instance.stub(:visible?).and_return(false)
        lambda{
          Panel.show @panel.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないコマを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Panel.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    it '指定のコマを返す' do
      Panel.any_instance.stub(:own?).and_return(true)
      pl = Panel.edit @panel.id, @author
      pl.should eq @panel
    end
    context '他人のコマを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        Panel.any_instance.stub(:own?).and_return(false)
        lambda{
          Panel.edit @panel.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないコマを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Panel.edit 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '関連テーブルプションに於いて' do
    context 'オプションがないとき' do
      it '3つの項目を含んでいる' do
        r = Panel.show_include_opt
        r.should have(3).items
      end
      it 'コマ絵を含んでいる' do
        r = Panel.show_include_opt
        r.has_key?(:panel_pictures).should be_true
      end
        it 'コマ絵は実素材を含んでいる' do
          r = Panel.show_include_opt
          r[:panel_pictures].has_key?(:picture).should be_true
        end
          it '実素材は絵師を含んでいる' do
            r = Panel.show_include_opt
            r[:panel_pictures][:picture].has_key?(:artist).should be_true
          end
          it '実素材はライセンスを含んでいる' do
            r = Panel.show_include_opt
            r[:panel_pictures][:picture].has_key?(:license).should be_true
          end
      it 'フキダシを含んでいる' do
        r = Panel.show_include_opt
        r.has_key?(:speech_balloons).should be_true
      end
        it 'フキダシはフキダシ枠を含んでいる' do
          r = Panel.show_include_opt
          r[:speech_balloons].has_key?(:balloons).should be_true
        end
        it 'フキダシはセリフを含んでいる' do
          r = Panel.show_include_opt
          r[:speech_balloons].has_key?(:speeches).should be_true
        end
      it '作家を含んでいる' do
        r = Panel.show_include_opt
        r.has_key?(:author).should be_true
      end
    end
    context 'オプションで原画を含ませたとき' do
      it '4つの項目を含んでいる' do
        r = Panel.show_include_opt(:include => {:test => {}})
        r.should have(4).items
      end
      it 'testを含んでいる' do
        r = Panel.show_include_opt(:include => {:test => {}})
        r.has_key?(:test).should be_true
      end
    end
  end
  describe 'json単体出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Panel.show_json_include_opt
      r.has_key?(:include).should be_true
    end
    it '3つの項目を含んでいる' do
      r = Panel.show_json_include_opt[:include]
      r.should have(3).items
    end
    it 'コマ絵を含んでいる' do
      r = Panel.show_json_include_opt[:include]
      r.has_key?(:panel_pictures).should be_true
    end
      it 'コマ絵は実素材を含んでいる' do
        r = Panel.show_json_include_opt[:include]
        r[:panel_pictures].has_key?(:picture).should be_true
      end
        it '実素材は絵師を含んでいる' do
          r = Panel.show_json_include_opt[:include]
          r[:panel_pictures][:picture].has_key?(:artist).should be_true
        end
        it '実素材はライセンスを含んでいる' do
          r = Panel.show_json_include_opt[:include]
          r[:panel_pictures][:picture].has_key?(:license).should be_true
        end
    it 'フキダシを含んでいる' do
      r = Panel.show_include_opt
      r.has_key?(:speech_balloons).should be_true
    end
      it 'フキダシはフキダシ枠を含んでいる' do
        r = Panel.show_include_opt
        r[:speech_balloons].has_key?(:balloons).should be_true
      end
      it 'フキダシはセリフを含んでいる' do
        r = Panel.show_include_opt
        r[:speech_balloons].has_key?(:speeches).should be_true
      end
    it '作家を含んでいる' do
      r = Panel.show_json_include_opt[:include]
      r.has_key?(:author).should be_true
    end
  end
  describe '一覧取得に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Panel.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Panel.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Panel.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Panel.page_size('7').should eq 7
      end
      it 'nilの場合はPanel.default_page_sizeになる' do
        Panel.page_size().should eq Panel.default_page_size
      end
      it '0以下の場合はPanel.default_page_sizeになる' do
        Panel.page_size('0').should eq Panel.default_page_size
      end
      it 'Panel.max_page_sizeを超えた場合はPanel.max_page_sizeになる' do
        Panel.page_size('1000').should eq Panel.max_page_size
      end
    end
    it 'リストを返す' do
      pl = Panel.list
      pl.should eq [@panel]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :panel, :author_id => @author.id, :updated_at => Time.now + 100
      pl = Panel.list
      pl.should eq [npl, @panel]
    end
    it '非公開のコマは含まない' do
      npl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      pl = Panel.list
      pl.should eq [@panel]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @npl2 = FactoryGirl.create :panel, :author_id => @author.id, :updated_at => Time.now + 100
        @npl3 = FactoryGirl.create :panel, :author_id => @author.id, :updated_at => Time.now + 200
        @npl4 = FactoryGirl.create :panel, :author_id => @author.id, :updated_at => Time.now + 300
        @npl5 = FactoryGirl.create :panel, :author_id => @author.id, :updated_at => Time.now + 400
        Panel.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pl = Panel.list
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = Panel.list( {}, 1)
        pl.should eq [@npl5, @npl4]
      end
      it 'page=2なら中間2件を返す' do
        pl = Panel.list({}, 2)
        pl.should eq [@npl3, @npl2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = Panel.list({}, 3)
        pl.should eq [@panel]
      end
    end
  end
  describe 'list関連テーブルプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Panel.list_opt
      r.has_key?(:include).should be_true
    end
    it '3つの項目を含んでいる' do
      r = Panel.list_opt[:include]
      r.should have(3).items
    end
    it 'コマ絵を含んでいる' do
      r = Panel.list_opt[:include]
      r.has_key?(:panel_pictures).should be_true
    end
      it 'コマ絵は実素材を含んでいる' do
        r = Panel.list_opt[:include]
        r[:panel_pictures].has_key?(:picture).should be_true
      end
        it '実素材は絵師を含んでいる' do
          r = Panel.list_opt[:include]
          r[:panel_pictures][:picture].has_key?(:artist).should be_true
        end
        it '実素材はライセンスを含んでいる' do
          r = Panel.list_opt[:include]
          r[:panel_pictures][:picture].has_key?(:license).should be_true
        end
    it 'フキダシを含んでいる' do
      r = Panel.show_include_opt
      r.has_key?(:speech_balloons).should be_true
    end
      it 'フキダシはフキダシ枠を含んでいる' do
        r = Panel.show_include_opt
        r[:speech_balloons].has_key?(:balloons).should be_true
      end
      it 'フキダシはセリフを含んでいる' do
        r = Panel.show_include_opt
        r[:speech_balloons].has_key?(:speeches).should be_true
      end
    it '作家を含んでいる' do
      r = Panel.list_opt[:include]
      r.has_key?(:author).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Panel.list_json_opt
      r.has_key?(:include).should be_true
    end
    it '3つの項目を含んでいる' do
      r = Panel.list_json_opt[:include]
      r.should have(3).items
    end
    it 'コマ絵を含んでいる' do
      r = Panel.list_json_opt[:include]
      r.has_key?(:panel_pictures).should be_true
    end
      it 'コマ絵は実素材を含んでいる' do
        r = Panel.list_json_opt[:include]
        r[:panel_pictures].has_key?(:picture).should be_true
      end
        it '実素材は絵師を含んでいる' do
          r = Panel.list_json_opt[:include]
          r[:panel_pictures][:picture].has_key?(:artist).should be_true
        end
        it '実素材はライセンスを含んでいる' do
          r = Panel.list_json_opt[:include]
          r[:panel_pictures][:picture].has_key?(:license).should be_true
        end
    it 'フキダシを含んでいる' do
      r = Panel.show_include_opt
      r.has_key?(:speech_balloons).should be_true
    end
      it 'フキダシはフキダシ枠を含んでいる' do
        r = Panel.show_include_opt
        r[:speech_balloons].has_key?(:balloons).should be_true
      end
      it 'フキダシはセリフを含んでいる' do
        r = Panel.show_include_opt
        r[:speech_balloons].has_key?(:speeches).should be_true
      end
    it '作家を含んでいる' do
      r = Panel.list_json_opt[:include]
      r.has_key?(:author).should be_true
    end
  end
  
  describe '自分のコマ一覧取得に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    it 'リストを返す' do
      pl = Panel.mylist @author
      pl.should eq [@panel]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :panel, :author_id => @author.id, :updated_at => Time.now + 100
      pl = Panel.mylist @author
      pl.should eq [npl, @panel]
    end
    it '他人のコマは公開でも含まない' do
      npl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      pl = Panel.mylist @author
      pl.should eq [@panel]
    end
    it '自分のコマは非公開でも含んでいる' do
      npl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0, :updated_at => Time.now + 100
      pl = Panel.mylist @author
      pl.should eq [npl, @panel]
    end
  end
  
  #異常データ検出
    #コマはコミックに従属しなくなったのでtで入れ替えるチェックは要らなくなった
    #コマ絵とフキダシの整合チェックをする
  describe 'id収集に於いて' do
    context 'つつがなく終わるとき' do
      it '第一パラメータで指定された配列中から第二引数のidを収集している' do
        a = [[{:panel_id => 1, :a => 'a'}, {:panel_id => 2, :a => 'a'}], 
          [{:panel_id => 3, :a => 'a'}, {:panel_id => 4, :a => 'a'}]]
        r = Panel.collect_element_value a, :panel_id
        r.should eq [1, 2, 3, 4]
      end
    end
    context 'Nil除外指示があるとき' do
      it '収集したpanel_idのうちnilは除外している' do
        a = [[{:panel_id => 1, :a => 'a'}, {:panel_id => 2, :a => 'a'}, {:panel_id => nil, :a => 'a'}], 
          [{:panel_id => 3, :a => 'a'}, {:panel_id => nil, :a => 'a'}, {:panel_id => 4, :a => 'a'}]]
        r = Panel.collect_element_value a, :panel_id, true
        r.should eq [1, 2, 3, 4]
      end
    end
  end
  describe 'id一致チェックに於いて' do
    #parent.idがNilのときはすべてNil。数値のときは全一致
    context '親が既存(idが数値)のとき' do
      it 'すべてid一致するときTrueを返す' do
        r = Panel.validate_id [1, 1, 1, 1], 1
        r.should be_true
      end
      it 'idが一致する中にNilが混じってもTrueを返す' do
        r = Panel.validate_id [1, 1, 1, 1, nil], 1
        r.should be_true
      end
      it 'すべて数値でも一致しないときFalseを返す' do
        r = Panel.validate_id [1, 1, 1, 1, 2], 1
        r.should be_false
      end
      it '数値とNilが混ざってもIDが一致しないときFalseを返す' do
        r = Panel.validate_id [1, 1, nil, 2], 1
        r.should be_false
      end
    end
    context '親が新規(idがNil)のとき' do
      it 'すべてNilはTrueを返す' do
        r = Panel.validate_id [ nil,nil,nil,nil], nil
        r.should be_true
      end
      it 'すべてnilでなければFalseを返す' do
        r = Panel.validate_id [nil,nil,nil,nil, 2], nil
        r.should be_false
      end
    end
  end
  describe 'idチェック単体に於いて' do
    before do
#      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 1
    end
    context 'つつがなく終わるとき' do
      it 'idを収集依頼している' do
        Panel.should_receive(:collect_element_value).with(any_args).exactly(1)
        Panel.stub(:collect_element_value).with(any_args).and_return([])
        Panel.stub(:validate_id).with(any_args).and_return(true)
        r = Panel.validate_element_id [], :panel_id, nil
      end
      it '収集したidを一致チェック依頼している' do
        Panel.should_receive(:validate_id).with(any_args).exactly(1)
        Panel.stub(:collect_element_value).with(any_args).and_return([])
        Panel.stub(:validate_id).with(any_args).and_return(true)
        r = Panel.validate_element_id [], :panel_id, nil
      end
    end
    #実データでチェック
    #依頼チェックだけでは不安なので最低限のチェックを
    context 'コマ新規のとき' do
      it 'コマ絵で正常通過している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => nil, :picture_id => @p.id, :t => nil)
        )
        r = Panel.validate_element_id [@panel.panel_pictures, @panel.speech_balloons], :panel_id, @panel.id
        r.should be_true
      end
      it 'フキダシで正常通過している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => nil, :speech_balloon_template_id => @sbt.id)
        )
        r = Panel.validate_element_id [@panel.panel_pictures, @panel.speech_balloons], :panel_id, @panel.id
        r.should be_true
      end
    end
    context 'コマ既存のとき' do
      it 'コマ絵で正常通過している' do
        @panel = FactoryGirl.create :panel, :author_id => @author.id
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => nil)
        )
        r = Panel.validate_element_id [@panel.panel_pictures, @panel.speech_balloons], :panel_id, @panel.id
        r.should be_true
      end
      it 'フキダシで正常通過している' do
        @panel = FactoryGirl.create :panel, :author_id => @author.id
        @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id)
        )
        r = Panel.validate_element_id [@panel.panel_pictures, @panel.speech_balloons], :panel_id, @panel.id
        r.should be_true
      end
    end
    context 'フキダシ新規のとき' do
      it 'バルーンで正常通過している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => nil, :speech_balloon_template_id => @sbt.id)
        )
        @panel.speech_balloons.first.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => nil)
        )
        r = Panel.validate_element_id [@panel.speech_balloons.first.speeches, @panel.speech_balloons.first.balloons], :speech_balloon_id, @panel.speech_balloons.first.id
        r.should be_true
      end
      it 'セリフで正常通過している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => nil, :speech_balloon_template_id => @sbt.id)
        )
        @panel.speech_balloons.first.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => nil)
        )
        r = Panel.validate_element_id [@panel.speech_balloons.first.speeches, @panel.speech_balloons.first.balloons], :speech_balloon_id, @panel.speech_balloons.first.id
        r.should be_true
      end
    end
  end
  describe 'idチェックリスト生成に於いて' do
    before do
      @panel = FactoryGirl.build :panel, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 'コマ部品とフキダシ部品のトータル2を返す' do
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => nil, :picture_id => @p.id, :t => nil)
        )
        @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => nil, :speech_balloon_template_id => @sbt.id)
        )
        @panel.speech_balloons.first.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => nil)
        )
        @panel.speech_balloons.first.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => nil)
        )
        r = @panel.validate_id_list
        r.should have(2).items
      end
      it 'コマ部品とフキダシ部品[複数:2]のケースでトータル3を返す' do
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => nil)
        )
        sb1 = @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id)
        )
        sb1.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => sb1.id)
        )
        sb1.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => sb1.id)
        )
        sb2 = @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id)
        )
        sb2.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => sb2.id)
        )
        sb2.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => sb2.id)
        )
        r = @panel.validate_id_list
        r.should have(3).items
      end
    end
  end
  describe 'idチェック複合に於いて' do
    before do
      @panel = FactoryGirl.build :panel, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 'コマ部品とフキダシ部品の二回チェック依頼している' do
        Panel.should_receive(:validate_element_id).with(any_args).exactly(2)
        Panel.stub(:validate_element_id).with(any_args).and_return(true)
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => nil)
        )
        sb1 = @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id)
        )
        sb1.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => sb1.id)
        )
        sb1.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => sb1.id)
        )
        r = Panel.validate_elements_id(@panel.validate_id_list)
      end
    end
  end
  describe 'シリアライズチェックに於いて' do
    context 'つつがなく終わるとき' do
      it '0からシリアライズされているならTrueを返す' do
        r = Panel.validate_t [0, 1, 2]
        r.should be_true
      end
      it '見た目はシリアライズされてなくてもソート結果が無事ならtrueを返す' do
        r = Panel.validate_t [0, 2, 1]
        r.should be_true
      end
      it '見た目はシリアライズされてなくてもソート結果が無事ならtrueを返す' do
        r = Panel.validate_t [ 2, 1, 4, 3, 0]
        r.should be_true
      end
    end
    context '異常なとき' do
      it '0から始まらないならFalseを返す' do
        r = Panel.validate_t [1, 2, 3]
        r.should be_false
      end
      it '連続していないならFalseを返す' do
        r = Panel.validate_t [0, 1, 2, 4]
        r.should be_false
      end
      it '連続していないならFalseを返す' do
        r = Panel.validate_t [0, 1, 2, 4, 5]
        r.should be_false
      end
    end
  end
  describe 'tチェック単体に於いて' do
    before do
    end
    context 'つつがなく終わるとき' do
      it 'コマ絵とフキダシのtを収集依頼している' do
        Panel.should_receive(:collect_element_value).with(any_args).exactly(1)
        Panel.stub(:collect_element_value).with(any_args).and_return([])
        Panel.stub(:validate_t).with(any_args).and_return(true)
        r = Panel.validate_element_t [], :t
      end
      it '収集したtをシリアライズチェック依頼している' do
        Panel.stub(:collect_element_value).with(any_args).and_return([])
        Panel.should_receive(:validate_t).with(any_args).exactly(1)
        Panel.stub(:validate_t).with(any_args).and_return(true)
        r = Panel.validate_element_t [], :t
      end
    end
    #実データでチェック
    #依頼チェックだけでは不安なので最低限のチェックを
    context '新規のとき' do
      it 'コマ絵で正常通過している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => nil, :picture_id => @p.id, :t => nil)
        )
        r = Panel.validate_element_t [@panel.panel_pictures, @panel.speech_balloons], :t
        r.should be_true
      end
      it 'フキダシで正常通過している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => nil, :speech_balloon_template_id => @sbt.id, :t => nil)
        )
        r = Panel.validate_element_t [@panel.panel_pictures, @panel.speech_balloons], :t
        r.should be_true
      end
    end
    context '既存のとき' do
      it 'コマ絵で正常通過している' do
        @panel = FactoryGirl.create :panel, :author_id => @author.id
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => 0)
        )
        r = Panel.validate_element_t [@panel.panel_pictures, @panel.speech_balloons], :t
        r.should be_true
      end
      it 'フキダシで正常通過している' do
        @panel = FactoryGirl.create :panel, :author_id => @author.id
        @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id, :t => 0)
        )
        r = Panel.validate_element_t [@panel.panel_pictures, @panel.speech_balloons], :t
        r.should be_true
      end
    end
  end
  describe '複合チェックに於いて' do
    context 'つつがなく終わるとき' do
      it 'している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => 0)
        )
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => 1)
        )
        sb1 = @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id, :t => 2)
        )
        sb1.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => sb1.id)
        )
        sb1.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => sb1.id)
        )
        sb2 = @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id, :t => 3)
        )
        sb2.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => sb2.id)
        )
        sb2.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => sb2.id)
        )
        r = @panel.validate_child
        r.should be_true
      end
    end
  end
  describe '複合チェックに於いて' do
    context 'つつがなく終わるとき' do
      it 'している' do
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => 0)
        )
        @panel.panel_pictures.build(
          FactoryGirl.attributes_for(:panel_picture, :panel_id => @panel.id, :picture_id => @p.id, :t => 1)
        )
        sb1 = @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id, :t => 2)
        )
        sb1.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => sb1.id)
        )
        sb1.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => sb1.id)
        )
        sb2 = @panel.speech_balloons.build(
          FactoryGirl.attributes_for(:speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @sbt.id, :t => 3)
        )
        sb2.balloons.build(
          FactoryGirl.attributes_for(:balloon, :speech_balloon_id => sb2.id)
        )
        sb2.speeches.build(
          FactoryGirl.attributes_for(:speech, :speech_balloon_id => sb2.id)
        )
        r = @panel.validate_child
        r.should be_true
      end
    end
  end
  describe '保存に於いて' do
    context 'つつがなく終わるとき' do
      it 'idチェックを依頼している' do
        Panel.should_receive(:validate_elements_id).with(any_args).exactly(1)
        Panel.stub(:validate_elements_id).with(any_args).and_return(true)
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        r = @panel.store
      end
      it '保存を依頼している' do
        Panel.stub(:validate_elements_id).with(any_args).and_return(true)
        Panel.any_instance.should_receive(:save).with(any_args).exactly(1)
        Panel.any_instance.stub(:save).with(any_args).and_return(true)
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        r = @panel.store
      end
      it 'Trueを返す' do
        Panel.stub(:validate_elements_id).with(any_args).and_return(true)
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        r = @panel.store
        r.should be_true
      end
      it 'エラーメッセージがセットされていない' do
        Panel.stub(:validate_elements_id).with(any_args).and_return(true)
        @panel = FactoryGirl.create :panel, :author_id => @author.id
        r = @panel.store
        @panel.errors.empty?.should be_true
      end
      context '新規のとき' do
        it 'Trueを返す' do
          Panel.stub(:validate_elements_id).with(any_args).and_return(true)
          @panel = FactoryGirl.build :panel, :author_id => @author.id
          r = @panel.store
          r.should be_true
        end
        it '行が追加されている' do
          Panel.stub(:validate_elements_id).with(any_args).and_return(true)
          @panel = FactoryGirl.build :panel, :author_id => @author.id
          lambda {
            r = @panel.store
          }.should change(Panel, :count)
        end
      end
      context '更新のとき' do
        it 'Trueを返す' do
          Panel.stub(:validate_elements_id).with(any_args).and_return(true)
          @panel = FactoryGirl.create :panel, :author_id => @author.id
          r = @panel.store
          r.should be_true
        end
        it '行が追加されていない' do
          Panel.stub(:validate_elements_id).with(any_args).and_return(true)
          @panel = FactoryGirl.create :panel, :author_id => @author.id
          lambda {
            r = @panel.store
          }.should_not change(Panel, :count)
        end
      end
    end
    context 'idチェックが失敗するとき' do
      it 'Falseを返す' do
        Panel.stub(:validate_elements_id).with(any_args).and_return(false)
        @panel = FactoryGirl.create :panel, :author_id => @author.id
        r = @panel.store
        r.should be_false
      end
      it 'エラーメッセージがセットされている' do
        Panel.stub(:validate_elements_id).with(any_args).and_return(false)
        @panel = FactoryGirl.create :panel, :author_id => @author.id
        r = @panel.store
        @panel.errors.empty?.should be_false
      end
      it '行が追加されていない' do
        Panel.stub(:validate_elements_id).with(any_args).and_return(false)
        @panel = FactoryGirl.build :panel, :author_id => @author.id
        lambda {
          r = @panel.store
        }.should_not change(Panel, :count)
      end
    end
  end
end
