# -*- encoding: utf-8 -*-
#素材
require 'spec_helper'

describe ResourcePicture do
  before do
    FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = @other_user.author
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @original_picture = FactoryGirl.create :original_picture, :artist_id => @artist.id
    
    class Mgk ; class Image ; end ; end
    Mgk::Image.stub(:from_blob).with(any_args).and_return([Mgk.new])
    Mgk.any_instance.stub(:format).with(any_args).and_return('png')
    Mgk.any_instance.stub(:format=).with(any_args)
    Mgk.any_instance.stub(:rows).with(any_args).and_return(200)
    Mgk.any_instance.stub(:columns).with(any_args).and_return(100)
    Mgk.any_instance.stub(:filesize).with(any_args).and_return(20000)
    Mgk.any_instance.stub(:to_blob).with(any_args).and_return('data')
    #原画ファイル削除だけは必ず成功するものとしておく
    PictureIO::LocalPicture.any_instance.stub(:delete).with(any_args).and_return(true)
    Picture.any_instance.stub(:subdirs).with(any_args).and_return(['', 'v', 'h', 'vh'])
    Picture.any_instance.stub(:h).with(any_args).and_return('data')
    Picture.any_instance.stub(:v).with(any_args).and_return('data')
    Picture.any_instance.stub(:vh).with(any_args).and_return('data')
  end
  
  describe '検証に於いて' do
    before do
      @p = FactoryGirl.create :picture, :original_picture_id => @original_picture.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.build :resource_picture, :artist_id => @artist.id, :original_picture_id => @original_picture.id, :license_id => @license.id, :picture_id => @p.id
    end
    
    it 'オーソドックスなデータなら通る' do
      @rp.should be_valid
    end
    
    context 'extを検証するとき' do
      it 'テストデータの確認' do
        @rp.ext = 'jpeg'
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.ext = ''
        @rp.should_not be_valid
      end
      it '5文字以上なら失敗する' do
        @rp.ext = 'a'*5
        @rp.should_not be_valid
      end
      it 'png,gif,jpeg以外なら失敗する' do
        @rp.ext = 'bmp'
        @rp.should_not be_valid
      end
    end
    context 'widthを検証するとき' do
      it 'テストデータの確認' do
        @rp.width = 1
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.width = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.width = 'a'
        @rp.should_not be_valid
      end
      it '0なら失敗する' do
        @rp.width = '0'
        @rp.should_not be_valid
      end
      it '負でも失敗する' do
        @rp.width = -1
        @rp.should_not be_valid
      end
    end
    context 'heightを検証するとき' do
      it 'テストデータの確認' do
        @rp.height = '1'
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.height = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.height = 'a'
        @rp.should_not be_valid
      end
      it '0なら失敗する' do
        @rp.height = '0'
        @rp.should_not be_valid
      end
      it '負でも失敗する' do
        @rp.height = -1
        @rp.should_not be_valid
      end
    end
    context 'filesizeを検証するとき' do
      it 'テストデータの確認' do
        @rp.filesize = '1'
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.filesize = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.filesize = 'a'
        @rp.should_not be_valid
      end
      it '負なら失敗する' do
        @rp.filesize = '-1'
        @rp.should_not be_valid
      end
      it '2MB以上なら失敗する' do
        @rp.filesize = 2000000+1
        @rp.should_not be_valid
      end
    end
    context 'artist_idを検証するとき' do
      it 'テストデータの確認' do
        @rp.artist_id = @artist.id
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.artist_id = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.artist_id = 'a'
        @rp.should_not be_valid
      end
      it '存在する絵師でなければ失敗する' do
        @rp.artist_id = 0
        @rp.should_not be_valid
      end
    end
    context 'license_idを検証するとき' do
      it 'テストデータの確認' do
        @rp.license_id = @license.id
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.license_id = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.license_id = 'a'
        @rp.should_not be_valid
      end
      it '存在する絵師でなければ失敗する' do
        @rp.license_id = 0
        @rp.should_not be_valid
      end
    end
    context 'original_picture_idを検証するとき' do
      it 'テストデータの確認' do
        @rp.original_picture_id = @original_picture.id
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.original_picture_id = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.original_picture_id = 'a'
        @rp.should_not be_valid
      end
      it '存在する原画でなければ失敗する' do
        @rp.original_picture_id = 0
        @rp.should_not be_valid
      end
    end
    context 'artist_nameを検証するとき' do
      it 'テストデータの確認' do
        @rp.artist_name = 'a'
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.artist_name = nil
        @rp.should_not be_valid
      end
    end
    context 'classnameを検証するとき' do
      it 'テストデータの確認' do
        @rp.classname = 'a'*50
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.classname = ''
        @rp.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @rp.classname = 'a'*51
        @rp.should_not be_valid
      end
    end
    context 'creditを検証するとき' do
      it 'テストデータの確認' do
        @rp.credit = 'a'
        @rp.should be_valid
      end
    end
    context 'settingsを検証するとき' do
      it 'テストデータの確認' do
        @rp.settings = 'a'
        @rp.should be_valid
      end
    end
    context 'picture_idを検証するとき' do
      it 'テストデータの確認' do
        @rp.picture_id = @p.id
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.picture_id = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.picture_id = 'a'
        @rp.should_not be_valid
      end
      it '存在する実素材でなければ失敗する' do
        @rp.picture_id = 0
        @rp.should_not be_valid
      end
    end
  end
  
  describe 'データ補充に於いて' do
    before do
    end
    
  end
  
  describe '単体取得に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :picture_id => @p.id
    end
    it '指定の素材を返す' do
      pic = ResourcePicture.show @rp.id
      pic.should eq @rp
    end
    context '関連テーブルオプションがないとき' do
      it 'ライセンスと絵師を含んでいる' do
        r = ResourcePicture.show_include_opt
        r.should eq [:license, :artist]
      end
    end
    context '関連テーブルオプションで原画を含ませたとき' do
      it 'ライセンスと絵師と原画データを含んでいる' do
        r = ResourcePicture.show_include_opt(:include => :original_picture)
        r.should eq [:license, :artist, :original_picture]
      end
    end
    context '存在しない素材を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          pic = ResourcePicture.show 0
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe 'json単体出力オプションに於いて' do
    it 'includeキーがライセンスと絵師を含んでいる' do
      r = ResourcePicture.show_json_include_opt
      r[:include].should eq [:license, :artist]
    end
  end
  describe '一覧取得に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :picture_id => @p.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        ResourcePicture.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        ResourcePicture.page().should eq 1
      end
      it '0以下の場合は1になる' do
        ResourcePicture.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        ResourcePicture.page_size('7').should eq 7
      end
      it 'nilの場合はResourcePicture.default_page_sizeになる' do
        ResourcePicture.page_size().should eq ResourcePicture.default_page_size
      end
      it '0以下の場合はResourcePicture.default_page_sizeになる' do
        ResourcePicture.page_size('0').should eq ResourcePicture.default_page_size
      end
      it 'ResourcePicture.max_page_sizeを超えた場合はResourcePicture.max_page_sizeになる' do
        ResourcePicture.page_size('1000').should eq ResourcePicture.max_page_size
      end
    end
    it 'リストを返す' do
      pic = ResourcePicture.list
      pic.should eq [@rp]
    end
    it '時系列で並んでいる' do
      nop = FactoryGirl.create :original_picture, :artist_id => @artist.id
      nrp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop.id, :picture_id => @p.id, :updated_at => Time.now + 100
      pic = ResourcePicture.list
      pic.should eq [nrp, @rp]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        nop2 = FactoryGirl.create :original_picture, :artist_id => @artist.id
        @nrp2 = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop2.id, :picture_id => @p.id, :updated_at => Time.now + 100
        nop3 = FactoryGirl.create :original_picture, :artist_id => @artist.id
        @nrp3 = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop3.id, :picture_id => @p.id, :updated_at => Time.now + 200
        nop4 = FactoryGirl.create :original_picture, :artist_id => @artist.id
        @nrp4 = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop4.id, :picture_id => @p.id, :updated_at => Time.now + 300
        nop5 = FactoryGirl.create :original_picture, :artist_id => @artist.id
        @nrp5 = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop5.id, :picture_id => @p.id, :updated_at => Time.now + 400
        ResourcePicture.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pic = ResourcePicture.list
        pic.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pic = ResourcePicture.list({}, 1)
        pic.should eq [@nrp5, @nrp4]
      end
      it 'page=2なら中間2件を返す' do
        pic = ResourcePicture.list({}, 2)
        pic.should eq [@nrp3, @nrp2]
      end
      it 'page=3なら先頭1件を返す' do
        pic = ResourcePicture.list({}, 3)
        pic.should eq [@rp]
      end
    end
  end
  describe 'json一覧出力オプションに於いて' do
    it 'includeキーがライセンスと絵師を含んでいる' do
      r = ResourcePicture.list_json_opt
      r[:include].should eq [:license, :artist]
    end
  end
  
  describe 'サイズの調整に於いて' do
    before do
    end
    
    it 'サイズに収まるときはそのまま使う' do
      ResourcePicture.fix_size_both(64, 64, 64, 64).should eq [64, 64]
    end
    it '小さくても拡大しない' do
      ResourcePicture.fix_size_both(64, 64, 32, 32).should eq [32, 32]
    end
    it '縦長のときは縦に合わせて縮小' do
      ResourcePicture.fix_size_both(64, 64, 64, 128).should eq [32, 64]
    end
    it '横長のときは横に合わせて縮小' do
      ResourcePicture.fix_size_both(64, 64, 128, 64).should eq [64, 32]
    end
  end
  
  describe 'フォーマット変換対象判定に於いて' do
    before do
      @rp = FactoryGirl.build :resource_picture, 
        :artist_id => @artist.id, :license_id => @license.id
    end
    context '変換するケース' do
      it '画像フォーマットがpngかつライセンスの変換禁止フラグが無効のときTrue' do
        ResourcePicture.any_instance.stub(:flag_gif_convert).with(any_args).and_return(0)
        @rp.ext = 'png'
        @rp.to_gif?.should be_true
      end
    end
    context '変換しないケース' do
      it '画像フォーマットがでない' do
        ResourcePicture.any_instance.stub(:flag_gif_convert).with(any_args).and_return(0)
        @rp.ext = 'gif'
        @rp.to_gif?.should be_false
      end
      it '変換禁止フラグが無効' do
        ResourcePicture.any_instance.stub(:flag_gif_convert).with(any_args).and_return(-1)
        @rp.ext = 'png'
        @rp.to_gif?.should be_false
      end
    end
  end
  
  describe 'Gifフォーマット変換に於いて' do
    before do
      Magick::Image.stub(:from_blob).with(any_args).and_return([Mgk.new])
    end
    context 'つつがなく終わるとき' do
      it 'Mgkオブジェクトが返る' do
        mgk = ResourcePicture.png_to_gif('mgkbin')
        mgk.is_a?(Mgk).should be_true
      end
      it 'Mgkオブジェクトはgif変換されている' do
        #スタブばかりで変換できないので代入されているかでチェックする
        Mgk.any_instance.should_receive(:format=).with('gif').exactly(1)
        ResourcePicture.png_to_gif('mgkbin')
      end
    end
    context 'RMagick変換が失敗したとき' do
      before do
        Magick::Image.should_receive(:from_blob).with(any_args).and_raise('StandardError')
      end
      it 'falseを返す' do
        res = ResourcePicture.png_to_gif('mgkbin')
        res.should be_false
      end
    end
  end
  
  describe '原画からのコピーデータセットに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :ext => 'jpeg', :width => 264, :height => 265, :filesize => 266, 
        :artist_id => @artist.id
      @rp = FactoryGirl.build :resource_picture, 
        :artist_id => @artist.id
    end
    it '原画オブジェクトから属性を取り出して対象素材にセットしている' do
      res = @rp.copy_data(@op)
      @rp.ext.should eq 'jpeg'
      @rp.width.should eq 264
      @rp.height.should eq 265
      @rp.filesize.should eq 266
      @rp.artist_id.should eq @artist.id
    end
  end
  
  describe 'RMagick変換に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', :credit => '{}', :settings => {}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
    end
    context 'つつがなく終わるとき' do
      it '画像データをオブジェクト化している' do
        Magick::Image.stub(:from_blob).and_return(['mgkobj'])
        @rp.data_to_mgk('mgkbin').should eq 'mgkobj'
      end
    end
    context '失敗するとき' do
      it 'Falseを返す' do
        Magick::Image.should_receive(:from_blob).with(any_args).and_raise('StandardError')
        @rp.data_to_mgk('mgkbin').should be_false
      end
      it '全体エラーがセットされている' do
        Magick::Image.should_receive(:from_blob).with(any_args).and_raise('StandardError')
        lambda {
          @rp.data_to_mgk('mgkbin')
        }.should change(@rp.errors[:base], :count)
      end
    end
  end
  describe '画像オブジェクトの取得に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', :credit => '{}', :settings => {}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
    end
    context '事前チェック' do
      before do
        OriginalPicture.any_instance.stub(:restore).with(any_args).and_return('data')
        ResourcePicture.any_instance.stub(:data_to_mgk).with(any_args).and_return(Mgk.new)
      end
      it '原画から画像データを取得している' do
        OriginalPicture.any_instance.should_receive(:restore).with(any_args).exactly(1)
        res = @rp.op_mgk
      end
      it '素材に原画データのRMagick変換を依頼している' do
        ResourcePicture.any_instance.should_receive(:data_to_mgk).with(any_args).exactly(1)
        res = @rp.op_mgk
      end
    end
    context 'つつがなく終わるとき' do
      #原画から画像データを取得する。
      #素材に原画データのRMagick変換を依頼し、画像オブジェクトを返す。 
      before do
        @mgk = Mgk.new
        OriginalPicture.any_instance.stub(:restore).with(any_args).and_return('data')
        ResourcePicture.any_instance.stub(:data_to_mgk).with(any_args).and_return(@mgk)
      end
      it '画像オブジェクトを返す' do
        res = @rp.op_mgk
        res.should eq @mgk
      end
    end
  end
  
  describe '新規実素材の取得に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', 
        :credit => '{}', :settings => {:reverse => 1, :gif_convert => 1}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
      @mgk = Mgk.new
    end
    context '事前チェック' do
      before do
        Picture.any_instance.stub(:copy_data).with(any_args).and_return(true)
        Picture.any_instance.stub(:store).with(any_args).and_return(true)
      end
      it '実素材に原画からのコピーデータを依頼している' do
        Picture.any_instance.should_receive(:copy_data).with(any_args).exactly(1)
        res = @rp.new_picture @mgk
      end
      it '実素材を保存している' do
        Picture.any_instance.should_receive(:store).with(any_args).exactly(1)
        res = @rp.new_picture @mgk
      end
    end
    context 'つつがなく終わるとき' do
      it '実素材に原画からのコピーデータをセットしている' do
        res = @rp.new_picture @mgk
        @rp.ext.should eq @op.ext
        @rp.width.should eq @op.width
        @rp.height.should eq @op.height
        @rp.filesize.should eq @op.filesize
        @rp.artist_id.should eq @op.artist_id
        @rp.original_picture_id.should eq @op.id
      end
      it '実素材を返す' do
        res = @rp.new_picture @mgk
        res.is_a?(Picture).should be_true
      end
    end
  end
  
  describe '作成・更新に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', :classname => 'StandardLicense', :credit => '{}', :settings => {}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
    end
    context '事前チェック' do
      before do
        #すべての処理が正常パターンで通過すれば、一番深い分岐まで通る。
        #それで外部のメソッド呼び出しだけに着目してテストする。
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:op_mgk).with(any_args).and_return(@mgk)
        ResourcePicture.any_instance.stub(:new_picture).with(any_args).and_return(@p)
        ResourcePicture.any_instance.stub(:store_picture_with_gif).with(any_args).and_return(true)
      end
      it '素材に原画からのコピーデータをセットしている' do
        ResourcePicture.any_instance.stub(:copy_data).with(any_args).and_return(true)
        ResourcePicture.any_instance.should_receive(:copy_data).with(any_args).exactly(1)
        res = @rp.store
      end
      it '画像オブジェクトの取得を依頼している' do
        ResourcePicture.any_instance.should_receive(:op_mgk).with(any_args).exactly(1)
        res = @rp.store
      end
      it '新規実素材の取得を依頼している' do
        ResourcePicture.any_instance.should_receive(:new_picture).with(any_args).exactly(1)
        res = @rp.store
      end
      it '自身を保存している' do
        ResourcePicture.any_instance.should_receive(:save).with(any_args).exactly(1)
        res = @rp.store
      end
      it '画像ファイルの作成・更新機能で画像を保存している' do
        ResourcePicture.any_instance.should_receive(:store_picture_with_gif).with(@mgk).exactly(1)
        res = @rp.store
      end
    end
    context 'つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:op_mgk).with(any_args).and_return(@mgk)
        ResourcePicture.any_instance.stub(:new_picture).with(any_args).and_return(@p)
        ResourcePicture.any_instance.stub(:store_picture_with_gif).with(any_args).and_return(true)
      end
      it 'Trueを返す' do
        res = @rp.store
        res.should be_true
      end
      it '実素材idをセットしている' do
        res = @rp.store
        @rp.picture_id.should_not be_nil
      end
      it '実素材idから最新画像idを取得してセットしている' do
        res = @rp.store
        @rp.picture_id.should eq @p.id
      end
      it '自身が保存されている' do
        lambda {
          res = @rp.store
        }.should change ResourcePicture, :count
      end
    end
    #以下から例外ケース。処理先頭から失敗させていく
    context '画像オブジェクトの取得に失敗したとき' do
      before do
        ResourcePicture.any_instance.stub(:op_mgk).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store
        res.should be_false
      end
      it '更新されていない' do
        @rp.store
        @rp.should be_a_new ResourcePicture
      end
    end
    context '新規実素材の取得に失敗したとき' do
      before do
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:op_mgk).with(any_args).and_return(@mgk)
        ResourcePicture.any_instance.stub(:new_picture).with(any_args).and_return(@p)
        Picture.any_instance.stub(:valid?).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store
        res.should be_false
      end
      it '更新されていない' do
        @rp.store
        @rp.should be_a_new ResourcePicture
      end
    end
    context '自身の保存に失敗したとき' do
      before do
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:op_mgk).with(any_args).and_return(@mgk)
        ResourcePicture.any_instance.stub(:new_picture).with(any_args).and_return(@p)
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store
        res.should be_false
      end
      it '更新されていない' do
        @rp.store
        @rp.should be_a_new ResourcePicture
      end
      it '処理を中断してロールバックする' do
        lambda {
          res = @rp.store
        }.should_not change Picture, :count
      end
    end
  end
  
  describe '画像ファイルの作成・更新に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', :credit => '{}', :settings => {}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
    end
    context '事前チェック' do
      before do
        #すべての処理が正常パターンで通過すれば、一番深い分岐まで通る。
        #それで外部のメソッド呼び出しだけに着目してテストする。
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
      end
      it '画像ファイルの作成・更新機能で画像を保存している' do
        ResourcePicture.any_instance.stub(:filename).with(any_args).and_return('1.png')
        ResourcePicture.any_instance.should_receive(:store_picture).with(@mgk, '1.png').exactly(1)
        res = @rp.store_picture_with_gif @mgk
      end
      it '自身にフォーマット変換対象かを問い合わせている' do
        ResourcePicture.any_instance.should_receive(:to_gif?).with(any_args).exactly(1)
        res = @rp.store_picture_with_gif @mgk
      end
      it '自身にGifフォーマット変換を依頼している' do
        ResourcePicture.should_receive(:png_to_gif).with(any_args).exactly(1)
        res = @rp.store_picture_with_gif @mgk
      end
      it '画像ファイルの作成・更新機能でgif画像を保存している' do
        #二回目の保存はgif変換の結果を渡す。
        ResourcePicture.any_instance.stub(:gifname).with(any_args).and_return('1.gif')
        ResourcePicture.any_instance.should_receive(:store_picture).with(@gifmgk, '1.gif').exactly(1)
        res = @rp.store_picture_with_gif @mgk
      end
    end
    context 'つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
      end
      it 'Trueを返す' do
        res = @rp.store_picture_with_gif @mgk
        res.should be_true
      end
    end
    context 'gif変換なしで、つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(false)
      end
      it 'Trueを返す' do
        res = @rp.store_picture_with_gif @mgk
        res.should be_true
      end
      it 'gif保存は呼ばれていない' do
        ResourcePicture.any_instance.should_receive(:store_picture).with(any_args).exactly(1)
        res = @rp.store_picture_with_gif @mgk
      end
    end
    #以下から例外ケース。処理先頭から失敗させていく
    context '画像の保存に失敗したとき' do
      before do
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store_picture_with_gif @mgk
        res.should be_false
      end
      it 'gif変換判定は呼ばれていない' do
        ResourcePicture.any_instance.should_not_receive(:to_gif?).with(any_args)
        res = @rp.store_picture_with_gif @mgk
      end
    end
    context 'gif変換に失敗したとき' do
      before do
        @mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store_picture_with_gif @mgk
        res.should be_false
      end
      it 'gif画像の保存は呼ばれていない' do
        #本画像の保存があるので、一度は呼ばれる
        ResourcePicture.any_instance.should_receive(:store_picture).with(any_args).exactly(1)
        res = @rp.store_picture_with_gif @mgk
      end
    end
    context 'gif画像の保存に失敗したとき' do
      before do
        ResourcePicture.any_instance.stub(:filename).with(any_args).and_return('1.png')
        ResourcePicture.any_instance.stub(:gifname).with(any_args).and_return('1.gif')
        @mgk = Mgk.new
        ResourcePicture.any_instance.stub(:store_picture).with(@mgk, '1.png').and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
        ResourcePicture.any_instance.stub(:store_picture).with(@gifmgk, '1.gif').and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store_picture_with_gif @mgk
        res.should be_false
      end
    end
  end
  
  describe '画像ファイルの作成・更新に於いて' do
    #PictureIo経由で画像を保存するための機能。ファイル名に自身のidを使うので事前に自身の保存が必要。
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', :credit => '{}', :settings => {}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
      @rp.copy_data @op
    end
    context '事前チェック' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        PictureIO.resource_picture_io.stub(:put).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:flag_thumbnail).with(any_args).and_return(1)
        ResourcePicture.any_instance.stub(:thumbnail).with(any_args).and_return('tmbdata')
      end
      it 'サムネイル化が1回呼ばれる' do
        ResourcePicture.any_instance.should_receive(:thumbnail).with(any_args).exactly(1)
        res = @rp.store_picture(Mgk.new, '1.gif')
      end
      it '画像ファイルの保存が2回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with(any_args).exactly(2)
        res = @rp.store_picture(Mgk.new, '1.gif')
      end
      it '画像ファイルのベースへのサムネイル保存が1回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with('tmbdata', '1.gif').exactly(1)
        res = @rp.store_picture(Mgk.new, '1.gif')
      end
      it '画像ファイルのfullへの保存が1回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with('data', '1.gif', 'full').exactly(1)
        res = @rp.store_picture(Mgk.new, '1.gif')
      end
    end
    context 'つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        PictureIO.resource_picture_io.stub(:put).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:flag_thumbnail).with(any_args).and_return(1)
      end
      it 'Trueを返す' do
#        res = @rp.store_picture(Mgk.new, '1.gif')
#        res.should be_true
      end
    end
    context '例外ケース' do
      before do
        ResourcePicture.any_instance.stub(:flag_thumbnail).with(any_args).and_return(1)
        PictureIO.resource_picture_io.stub(:put).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
 #       res = @rp.store_picture(Mgk.new, '1.gif')
 #       res.should be_false
      end
    end
    
  end
  
  describe 'サムネイル変換に於いて' do
    #サムネイル化した画像データを返すが、スタブをおくので、リサイズと画像データ化を試みるかをチェックする
    before do
      ResourcePicture.stub(:resize).with(any_args).and_return(Mgk.new)
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', :credit => '{}', :settings => {}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
    end
    it 'サムネイルサイズに縮小した画像データを返す' do
      ResourcePicture.should_receive(:resize).with(any_args).exactly(1)
      @rp.thumbnail(Mgk.new)
    end
    it 'データが返る' do
      #全体スタブより
      @rp.thumbnail(Mgk.new).should eq 'data'
    end
  end
  
  describe 'クレジットデータに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      attr = {:original_picture_id => @op.id, :license_id => @license.id, :artist_name => 'tester', :credit => '{"system_picture_id": 2}', :settings => {}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
    end
    it 'system_picture_idが入っている' do
      res = @rp.credit_data
      res["system_picture_id"].should eq 2
    end
  end
  
end

